@@ -25,6 +25,10 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
     if (missing(file)) {
         stop("Model file name missing")
     }
+    if (!file.exists(file)) {
+        stop(paste("Model file \"", file, "\" not found", sep=""))
+    }
+    
     p <- .Call("make_console", PACKAGE="rjags") 
     .Call("check_model", p, file, PACKAGE="rjags")
 
@@ -63,18 +67,32 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
 
         checkParameters <- function(inits) {
             if(!is.list(inits))
-              return (FALSE)
-            if (is.null(names(inits)) || any(nchar(names(inits)) == 0))
-              return (FALSE)
+                return (FALSE)
+
+            inames <- names(inits)
+            if (is.null(inames) || any(nchar(inames) == 0))
+                return (FALSE)
+
+            if (any(duplicated(inames)))
+                return (FALSE)
+            
+            if (any(inames==".RNG.name")) {
+                rngname <- inits[[".RNG.name"]]
+                if (!is.character(rngname) || length(rngname) != 1)
+                    return (FALSE)
+                inits[[".RNG.name"]] <- NULL
+            }
+
             if (!all(sapply(inits, is.numeric)))
-              return (FALSE)
+                return (FALSE)
             
             return (TRUE)
         }
         
         setParameters <- function(inits, chain) {
             if (!is.null(inits[[".RNG.name"]])) {
-                .Call("set_rng_name", p, inits[[".RNG.name"]], PACKAGE="rjags")
+                .Call("set_rng_name", p, inits[[".RNG.name"]],
+                      as.integer(chain), PACKAGE="rjags")
                 inits[[".RNG.name"]] <- NULL
             }
             .Call("set_parameters", p, inits, as.integer(chain),
@@ -161,7 +179,7 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
 
                     if (interactive()) {
                       #Show progress bar
-                      pb <- txtProgressBar(0, niter, style=1,width=50,
+                      pb <- txtProgressBar(0, niter, style=3,width=50,
                                            char=ifelse(adapting,"+","*"))
                       n <- niter
                       while (n > 0) {
@@ -212,7 +230,7 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
                               .Call("set_parameters", p, statei, i, PACKAGE="rjags")
                           }
                           .Call("initialize", p, PACKAGE="rjags")
-                          #Redo adaptation
+                          ## Redo adaptation
                           cat("Adapting\n")
                           .Call("update", p, n.adapt, TRUE, PACKAGE="rjags")
                           model.state <<- .Call("get_state", p, PACKAGE="rjags")
@@ -224,6 +242,73 @@ jags.model <- function(file, data=sys.frame(sys.parent()), inits,
     return(model)
 }
 
+parse.varname <- function(varname) {
+
+  ## Try to parse string of form "a" or "a[n,p:q,r]" where "a" is a
+  ## variable name and n,p,q,r are integers
+
+  v <- try(parse(text=varname, n=1), silent=TRUE)
+  if (!is.expression(v) || length(v) != 1)
+    return(NULL)
+
+  v <- v[[1]]
+  if (is.name(v)) {
+    ##Full node array requested
+    return(list(name=deparse(v)))
+  }
+  else if (is.call(v) && identical(deparse(v[[1]]), "[") && length(v) > 2) {
+    ##Subset requested
+    ndim <- length(v) - 2
+    lower <- upper <- numeric(ndim)
+    if (any(nchar(sapply(v, deparse)) == 0)) {
+      ##We have to catch empty indices here or they will cause trouble
+      ##below
+      return(NULL)
+    }
+    for (i in 1:ndim) {
+      index <- v[[i+2]]
+      if (is.numeric(index)) {
+        ##Single index
+        lower[i] <- upper[i] <- index
+      }
+      else if (is.call(index) && length(index) == 3 &&
+               identical(deparse(index[[1]]), ":") &&
+               is.numeric(index[[2]]) && is.numeric(index[[3]]))
+        {
+          ##Index range
+          lower[i] <- index[[2]]
+          upper[i] <- index[[3]]
+        }
+      else return(NULL)
+    }
+    if (any(upper < lower))
+      return (NULL)
+    return(list(name = deparse(v[[2]]), lower=lower, upper=upper))
+  }
+  return(NULL)
+}
+
+parse.varnames <- function(varnames)
+{
+  names <- character(length(varnames))
+  lower <- upper <- vector("list", length(varnames))
+  for (i in seq(along=varnames)) {
+    y <- parse.varname(varnames[i])
+    if (is.null(y)) {
+      stop(paste("Invalid variable subset", varnames[i]))
+    }
+    names[i] <- y$name
+    if (!is.null(y$lower)) {
+      lower[[i]] <- y$lower
+    }
+    if (!is.null(y$upper)) {
+      upper[[i]] <- y$upper
+    }
+  }
+  return(list(names=names, lower=lower, upper=upper))
+}
+
+
 jags.samples <-
   function(model, variable.names=NULL, n.iter, thin=1, type="trace")
 {
@@ -245,8 +330,9 @@ jags.samples <-
               type, PACKAGE="rjags")
     }
     else {
-        .Call("set_monitors", model$ptr(), variable.names,
-              as.integer(thin), type, PACKAGE="rjags")
+      pn <- parse.varnames(variable.names)
+      .Call("set_monitors", model$ptr(), pn$names, pn$lower, pn$upper,
+            as.integer(thin), type, PACKAGE="rjags")
     }
     update(model, n.iter)
     ans <- .Call("get_monitored_values", model$ptr(), type, PACKAGE="rjags")
@@ -258,8 +344,8 @@ jags.samples <-
     }
     else {
         for (i in seq(along=variable.names)) {
-            .Call("clear_monitor", model$ptr(), variable.names[i], type,
-                  PACKAGE="rjags")
+            .Call("clear_monitor", model$ptr(), pn$names[i], pn$lower[[i]],
+                  pn$upper[[i]], type, PACKAGE="rjags")
         }
     }
     return(ans)
@@ -273,15 +359,41 @@ list.samplers <- function(model)
     .Call("get_samplers", model$ptr(), PACKAGE="rjags")
 }
 
+
 coda.names <- function(basename, dim)
 {
+    ## Utility function used to get the names of the individual elements
+    ## of a node array
+  
     if (prod(dim) == 1)
       return(basename)
+
+    ##Default lower and upper limits
+    ndim <- length(dim)
+    lower <- rep(1, ndim)
+    upper <- dim
+
+    ##If the node name is a subset, we try to parse it to get the
+    ##names of its elements. For example, if basename is "A[2:3]"
+    ##we want to return names "A[2]", "A[3]" not "A[2:3][1]", "A[2:3][2]".
+    pn <- parse.varname(basename)
+    if (!is.null(pn) && !is.null(pn$lower) && !is.null(pn$upper)) {
+        if (length(pn$lower) == length(pn$upper)) {
+            dim2 <- pn$upper - pn$lower + 1
+            if (isTRUE(all.equal(dim[dim!=1], dim2[dim2!=1],
+                                 check.attributes=FALSE))) {
+                basename <- pn$name
+                lower <- pn$lower
+                upper <- pn$upper
+                ndim <- length(dim2)
+            }
+        }
+    }
     
-    indices <- as.character(1:dim[1])
-    if (length(dim) > 1) {
-        for (i in 2:length(dim)) {
-            indices <- outer(indices, 1:dim[i], FUN=paste, sep=",")
+    indices <- as.character(lower[1]:upper[1])
+    if (ndim > 1) {
+        for (i in 2:ndim) {
+            indices <- outer(indices, lower[i]:upper[i], FUN=paste, sep=",")
         }
     }
     paste(basename,"[",as.vector(indices),"]",sep="")
